import {
  TIME_BEFORE_EXPIRING_JWT_IN_SECONDS
} from "./chunk-BMYZMMVR.js";
import {
  deleteCookie,
  getCookie,
  setCookie
} from "./chunk-5JXPQKM4.js";
import {
  getJWTExpiration
} from "./chunk-MLMLBCSI.js";
import {
  __async,
  __spreadProps,
  __spreadValues
} from "./chunk-JPRRZPBL.js";

// src/CrossmintAuthClient.ts
import {
  AUTH_SDK_ROOT_ENDPOINT,
  CROSSMINT_API_VERSION,
  CrossmintAuth,
  REFRESH_TOKEN_PREFIX,
  SESSION_PREFIX
} from "@crossmint/common-sdk-auth";
import { queueTask } from "@crossmint/client-sdk-base";
var CrossmintAuthClient = class _CrossmintAuthClient extends CrossmintAuth {
  constructor(crossmint, apiClient, config = {}) {
    var _a, _b;
    super(crossmint, apiClient, config);
    this.refreshTask = null;
    this.isRefreshing = false;
    this.callbacks = (_a = config.callbacks) != null ? _a : {};
    this.logoutRoute = (_b = config.logoutRoute) != null ? _b : null;
  }
  static from(crossmint, config = {}) {
    const authClient = new _CrossmintAuthClient(crossmint, CrossmintAuth.defaultApiClient(crossmint), config);
    if (typeof window !== "undefined") {
      authClient.handleRefreshAuthMaterial();
    }
    return authClient;
  }
  getUser() {
    return __async(this, null, function* () {
      const result = yield this.apiClient.get(`api/${CROSSMINT_API_VERSION}/sdk/auth/user`, {
        headers: {
          "Content-Type": "application/json"
        }
      });
      const user = yield result.json();
      return user;
    });
  }
  storeAuthMaterial(authMaterial) {
    setCookie(SESSION_PREFIX, authMaterial.jwt);
    setCookie(REFRESH_TOKEN_PREFIX, authMaterial.refreshToken.secret, authMaterial.refreshToken.expiresAt);
  }
  logout() {
    return __async(this, null, function* () {
      var _a, _b;
      try {
        if (this.logoutRoute != null) {
          yield this.logoutFromCustomRoute();
        } else {
          yield this.logoutFromDefaultRoute(getCookie(REFRESH_TOKEN_PREFIX));
        }
      } catch (error) {
        console.error(error);
      } finally {
        deleteCookie(REFRESH_TOKEN_PREFIX);
        deleteCookie(SESSION_PREFIX);
        (_b = (_a = this.callbacks).onLogout) == null ? void 0 : _b.call(_a);
      }
    });
  }
  handleRefreshAuthMaterial(refreshTokenSecret) {
    return __async(this, null, function* () {
      var _a, _b;
      const refreshToken = refreshTokenSecret != null ? refreshTokenSecret : getCookie(REFRESH_TOKEN_PREFIX);
      if (refreshToken == null && this.refreshRoute == null || this.isRefreshing) {
        return;
      }
      try {
        this.isRefreshing = true;
        const authMaterial = yield this.refreshAuthMaterial(refreshToken);
        if (this.refreshRoute == null) {
          this.storeAuthMaterial(authMaterial);
        }
        (_b = (_a = this.callbacks).onTokenRefresh) == null ? void 0 : _b.call(_a, authMaterial);
        this.scheduleNextRefresh(authMaterial.jwt);
      } catch (error) {
        console.error(error);
        this.logout();
      } finally {
        this.isRefreshing = false;
      }
    });
  }
  getOAuthUrl(provider) {
    return __async(this, null, function* () {
      const result = yield this.apiClient.get(`${AUTH_SDK_ROOT_ENDPOINT}/social/${provider}/start`, {
        headers: {
          "Content-Type": "application/json"
        }
      });
      const data = yield result.json();
      return data.oauthUrl;
    });
  }
  sendEmailOtp(email) {
    return __async(this, null, function* () {
      const result = yield this.apiClient.post(`${AUTH_SDK_ROOT_ENDPOINT}/otps/send`, {
        headers: {
          "Content-Type": "application/json"
        },
        body: JSON.stringify({ email })
      });
      return yield result.json();
    });
  }
  confirmEmailOtp(email, emailId, token) {
    return __async(this, null, function* () {
      const queryParams = new URLSearchParams({
        email,
        signinAuthenticationMethod: "email",
        token,
        locale: "en",
        state: emailId,
        callbackUrl: `${this.apiClient.baseUrl}/${AUTH_SDK_ROOT_ENDPOINT}/we-dont-actually-use-this-anymore`
      });
      const result = yield this.apiClient.post(`${AUTH_SDK_ROOT_ENDPOINT}/authenticate?${queryParams}`, {
        headers: {
          "Content-Type": "application/json"
        }
      });
      const resData = yield result.json();
      const callbackUrl = new URL(resData.callbackUrl);
      return callbackUrl.searchParams.get("oneTimeSecret");
    });
  }
  signInWithFarcaster(data) {
    return __async(this, null, function* () {
      const queryParams = new URLSearchParams({
        signinAuthenticationMethod: "farcaster",
        callbackUrl: `${this.apiClient.baseUrl}/${AUTH_SDK_ROOT_ENDPOINT}/callback?isPopup=false`
      });
      const result = yield this.apiClient.post(`${AUTH_SDK_ROOT_ENDPOINT}/authenticate?${queryParams}`, {
        headers: {
          "Content-Type": "application/json"
        },
        body: JSON.stringify(__spreadProps(__spreadValues({}, data), {
          domain: data.signatureParams.domain,
          redirect: true,
          callbackUrl: `${this.apiClient.baseUrl}/${AUTH_SDK_ROOT_ENDPOINT}/callback?isPopup=false`
        }))
      });
      const resData = yield result.json();
      const callbackUrl = new URL(resData.callbackUrl);
      return callbackUrl.searchParams.get("oneTimeSecret");
    });
  }
  signInWithSmartWallet(address) {
    return __async(this, null, function* () {
      const queryParams = new URLSearchParams({
        signinAuthenticationMethod: "evm"
      });
      const result = yield this.apiClient.post(
        `${AUTH_SDK_ROOT_ENDPOINT}/crypto_wallets/authenticate/start?${queryParams}`,
        {
          headers: {
            "Content-Type": "application/json"
          },
          body: JSON.stringify({ walletAddress: address })
        }
      );
      return yield result.json();
    });
  }
  authenticateSmartWallet(address, signature) {
    return __async(this, null, function* () {
      const queryParams = new URLSearchParams({
        signinAuthenticationMethod: "evm",
        callbackUrl: `${this.apiClient.baseUrl}/${AUTH_SDK_ROOT_ENDPOINT}/we-dont-actually-use-this-anymore`
      });
      const result = yield this.apiClient.post(
        `${AUTH_SDK_ROOT_ENDPOINT}/crypto_wallets/authenticate?${queryParams}`,
        {
          headers: {
            "Content-Type": "application/json"
          },
          body: JSON.stringify({ walletAddress: address, signature })
        }
      );
      return yield result.json();
    });
  }
  logoutFromCustomRoute() {
    return __async(this, null, function* () {
      if (!this.logoutRoute) {
        throw new Error("Custom logout route is not set");
      }
      return yield fetch(this.logoutRoute, { method: "POST" });
    });
  }
  scheduleNextRefresh(jwt) {
    const jwtExpiration = getJWTExpiration(jwt);
    if (!jwtExpiration) {
      throw new Error("Invalid JWT");
    }
    const currentTime = Date.now() / 1e3;
    const timeToExpire = jwtExpiration - currentTime - TIME_BEFORE_EXPIRING_JWT_IN_SECONDS;
    if (timeToExpire > 0) {
      const endTime = Date.now() + timeToExpire * 1e3;
      this.cancelScheduledRefresh();
      this.refreshTask = queueTask(() => this.handleRefreshAuthMaterial(), endTime);
    }
  }
  cancelScheduledRefresh() {
    if (this.refreshTask) {
      this.refreshTask.cancel();
      this.refreshTask = null;
    }
  }
};

export {
  CrossmintAuthClient
};
//# sourceMappingURL=chunk-7YATBCAU.js.map